# -----------------------------------------------------------------
# lvr's cmake configuration
#
# provided options:
# - LVR2_USE_STATIC_LIBS(=OFF) to link the static libraries of lvr
#
# provided interface variables:
# - LVR2_INCLUDE_DIRS
# - LVR2_DEFINITIONS
# - LVR2_LIBRARIES
#
# Do not forget to add_defintions(${LVR2_DEFINITIONS}) as they
# describe the build configuration of liblvr.
#
# -----------------------------------------------------------------

####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was LVR2Config.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################
include("${CMAKE_CURRENT_LIST_DIR}/lvr2Targets.cmake")

list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}/Modules")

cmake_policy(PUSH)
cmake_policy(SET CMP0012 NEW)

set(LVR2_INCLUDE_DIRS /opt/ros/melodic/include/lvr2/ext/nanoflann;/opt/ros/melodic/include/lvr2/ext/psimpl;/opt/ros/melodic/include/lvr2/ext/rply;/opt/ros/melodic/include/lvr2/ext/laslib;/opt/ros/melodic/include/lvr2/ext/slam6d;/opt/ros/melodic/include/lvr2/ext/CTPL;/opt/ros/melodic/include)
set(LVR2_DEFINITIONS -DLVR2_USE_OPENCL;-DLVR2_USE_OPEN_MP)

option(LVR2_USE_STATIC_LIBS OFF)
if(LVR2_USE_STATIC_LIBS)
  set(LVR2_LIBRARIES lvr2_static)
else()
  set(LVR2_LIBRARIES lvr2)
endif()

find_package(OpenCV REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${OpenCV_INCLUDE_DIRS})

find_package(Eigen3 REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${EIGEN3_INCLUDE_DIR})

find_package(VTK REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${VTK_INCLUDE_DIRS})
list(APPEND LVR2_DEFINITIONS ${VTK_DEFINTIONS})

# nabo
if(FALSE)
  find_package(Nabo REQUIRED)
  list(APPEND LVR2_INCLUDE_DIRS ${NABO_INCLUDE_DIRS})
endif()

# pcl
if()
  find_package(PCL REQUIRED)
  list(APPEND LVR2_INCLUDE_DIRS ${PCL_INCLUDE_DIRS})
  list(APPEND LVR2_DEFINITIONS ${PCL_DEFINITIONS})
endif()

# cgal
if(0)
  find_package(CGAL REQUIRED)
  list(APPEND LVR2_INCLUDE_DIRS ${CGAL_INCLUDE_DIRS})
endif()

find_package(Boost REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${Boost_INCLUDE_DIRS})
list(APPEND LVR2_DEFINITIONS ${Boost_LIB_DIAGNOSTIC_DEFINITIONS})

find_package(OpenGL REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${OPENGL_INCLUDE_DIR})

find_package(GLUT REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${GLUT_INCLUDE_DIR})

find_package(Lz4 REQUIRED)
list(APPEND LVR2_INCLUDE_DIRS ${LZ4_INCLUDE_DIR})
list(APPEND LVR2_LIBRARIES ${LZ4_LIBRARY})

# libfreenect
if()
  find_package(PkgConfig REQUIRED)
  pkg_check_modules(LIBFREENECT REQUIRED libfreenect)
  list(APPEND LVR2_INCLUDE_DIRS ${LIBFREENECT_INCLUDE_DIRS})
endif()

# stann
if(FALSE)
  set(STANN_DIR )
  find_package(STANN REQUIRED)
  list(APPEND LVR2_2INCLUDE_DIRS ${STANN_INCLUDE_DIR})
endif()

# HDF5
find_package(HDF5 REQUIRED COMPONENTS C CXX HL)
list(APPEND LVR2_INCLUDE_DIRS ${HDF5_INCLUDE_DIRS})
list(APPEND LVR2_LIBRARIES ${HDF5_LIBRARIES} ${HDF5_HL_LIBRARIES})

# Embree
if(0)
    set_property(GLOBAL PROPERTY FIND_LIBRARY_USE_LIB64_PATHS ON)
    find_package(embree 3.0 REQUIRED)
    list(APPEND LVR2_INCLUDE_DIRS ${EMBREE_INCLUDE_DIRS})
    list(APPEND LVR2_LIBRARIES ${EMBREE_LIBRARY})
endif()

cmake_policy(POP)

set(LVR2_FOUND TRUE)
